@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2011 Andrew M. Bishop
+ This file Copyright 2008-2012 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -22,20 +22,16 @@
 
 #include <assert.h>
 #include <stdlib.h>
-#include <stdio.h>
 #include <string.h>
-#include <sys/stat.h>
 
 #include "types.h"
 #include "nodes.h"
-#include "segments.h"
 
+#include "typesx.h"
 #include "nodesx.h"
 #include "segmentsx.h"
 #include "waysx.h"
 
-#include "types.h"
-
 #include "files.h"
 #include "logging.h"
 #include "sorting.h"
@@ -55,7 +51,7 @@ static int sort_by_id(NodeX *a,NodeX *b);
 static int deduplicate_and_index_by_id(NodeX *nodex,index_t index);
 
 static int sort_by_lat_long(NodeX *a,NodeX *b);
-static int index_by_lat_long(NodeX *nodex,index_t index);
+static int delete_pruned_and_index_by_lat_long(NodeX *nodex,index_t index);
 
 
 /*++++++++++++++++++++++++++++++++++++++
@@ -200,7 +196,7 @@ void SortNodeList(NodesX *nodesx)
 
  sortnodesx=nodesx;
 
- filesort_fixed(nodesx->fd,fd,sizeof(NodeX),(int (*)(const void*,const void*))sort_by_id,(int (*)(void*,index_t))deduplicate_and_index_by_id);
+ nodesx->number=filesort_fixed(nodesx->fd,fd,sizeof(NodeX),(int (*)(const void*,const void*))sort_by_id,(int (*)(void*,index_t))deduplicate_and_index_by_id);
 
  /* Close the files */
 
@@ -253,8 +249,6 @@ static int deduplicate_and_index_by_id(NodeX *nodex,index_t index)
    {
     sortnodesx->idata[index]=nodex->id;
 
-    sortnodesx->number++;
-
     return(1);
    }
  else
@@ -275,10 +269,11 @@ static int deduplicate_and_index_by_id(NodeX *nodex,index_t index)
 void SortNodeListGeographically(NodesX *nodesx)
 {
  int fd;
+ index_t kept;
 
  /* Print the start message */
 
- printf_first("Sorting Nodes Geographically");
+ printf_first("Sorting Nodes Geographically (Deleting Pruned)");
 
  /* Allocate the memory for the geographical index array */
 
@@ -298,7 +293,7 @@ void SortNodeListGeographically(NodesX *nodesx)
 
  sortnodesx=nodesx;
 
- filesort_fixed(nodesx->fd,fd,sizeof(NodeX),(int (*)(const void*,const void*))sort_by_lat_long,(int (*)(void*,index_t))index_by_lat_long);
+ kept=filesort_fixed(nodesx->fd,fd,sizeof(NodeX),(int (*)(const void*,const void*))sort_by_lat_long,(int (*)(void*,index_t))delete_pruned_and_index_by_lat_long);
 
  /* Close the files */
 
@@ -307,7 +302,8 @@ void SortNodeListGeographically(NodesX *nodesx)
 
  /* Print the final message */
 
- printf_last("Sorted Nodes Geographically");
+ printf_last("Sorted Nodes Geographically: Nodes=%"Pindex_t" Deleted=%"Pindex_t,kept,nodesx->number-kept);
+ nodesx->number=kept;
 }
 
 
@@ -325,54 +321,69 @@ void SortNodeListGeographically(NodesX *nodesx)
 
 static int sort_by_lat_long(NodeX *a,NodeX *b)
 {
- ll_bin_t a_lon=latlong_to_bin(a->longitude);
- ll_bin_t b_lon=latlong_to_bin(b->longitude);
+ int a_pruned=IsPrunedNodeX(a);
+ int b_pruned=IsPrunedNodeX(b);
 
- if(a_lon<b_lon)
-    return(-1);
- else if(a_lon>b_lon)
+ if(a_pruned && b_pruned)
+    return(0);
+ else if(a_pruned)
     return(1);
+ else if(b_pruned)
+    return(-1);
  else
    {
-    ll_bin_t a_lat=latlong_to_bin(a->latitude);
-    ll_bin_t b_lat=latlong_to_bin(b->latitude);
+    ll_bin_t a_lon=latlong_to_bin(a->longitude);
+    ll_bin_t b_lon=latlong_to_bin(b->longitude);
 
-    if(a_lat<b_lat)
+    if(a_lon<b_lon)
        return(-1);
-    else if(a_lat>b_lat)
+    else if(a_lon>b_lon)
        return(1);
     else
       {
-       if(a->longitude<b->longitude)
+       ll_bin_t a_lat=latlong_to_bin(a->latitude);
+       ll_bin_t b_lat=latlong_to_bin(b->latitude);
+
+       if(a_lat<b_lat)
           return(-1);
-       else if(a->longitude>b->longitude)
+       else if(a_lat>b_lat)
           return(1);
        else
          {
-          if(a->latitude<b->latitude)
+          if(a->longitude<b->longitude)
              return(-1);
-          else if(a->latitude>b->latitude)
+          else if(a->longitude>b->longitude)
              return(1);
-         }
+          else
+            {
+             if(a->latitude<b->latitude)
+                return(-1);
+             else if(a->latitude>b->latitude)
+                return(1);
+            }
 
-       return(0);
+          return(0);
+         }
       }
    }
 }
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Create the index between the sorted and unsorted nodes.
+  Delete the pruned nodes and create the index between the sorted and unsorted nodes.
 
-  int index_by_lat_long Return 1 if the value is to be kept, otherwise 0.
+  int delete_pruned_and_index_by_lat_long Return 1 if the value is to be kept, otherwise 0.
 
   NodeX *nodex The extended node.
 
   index_t index The index of this node in the total.
   ++++++++++++++++++++++++++++++++++++++*/
 
-static int index_by_lat_long(NodeX *nodex,index_t index)
+static int delete_pruned_and_index_by_lat_long(NodeX *nodex,index_t index)
 {
+ if(IsPrunedNodeX(nodex))
+    return(0);
+
  /* Create the index from the previous sort to the current one */
 
  sortnodesx->gdata[nodex->id]=index;
@@ -531,14 +542,6 @@ void RemoveNonHighwayNodes(NodesX *nodesx,SegmentsX *segmentsx)
  free(segmentsx->usednode);
  segmentsx->usednode=NULL;
 
- /* Allocate and set the super-node markers */
-
- nodesx->super=(uint8_t*)malloc((1+nodesx->number/8)*sizeof(uint8_t));
-
- assert(nodesx->super); /* Check calloc() worked */
-
- memset(nodesx->super,~0,(1+nodesx->number/8));
-
  /* Print the final message */
 
  printf_last("Checked Nodes: Nodes=%"Pindex_t" Highway=%"Pindex_t" not-Highway=%"Pindex_t,total,highway,nothighway);
@@ -594,6 +597,11 @@ void UpdateNodes(NodesX *nodesx,SegmentsX *segmentsx)
  nodesx->fd=CloseFile(nodesx->fd);
  CloseFile(fd);
 
+ /* Free the memory */
+
+ free(nodesx->super);
+ nodesx->super=NULL;
+
  /* Print the final message */
 
  printf_last("Updated Super Nodes: Nodes=%"Pindex_t,nodesx->number);
@@ -642,7 +650,7 @@ void SaveNodeList(NodesX *nodesx,const char *filename)
  for(i=0;i<nodesx->number;i++)
    {
     NodeX nodex;
-    Node node;
+    Node node={0};
     ll_bin_t latbin,lonbin;
     ll_bin2_t llbin;
 
