@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2011 Andrew M. Bishop
+ This file Copyright 2008-2012 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -27,13 +27,13 @@
 #include <ctype.h>
 
 #include "typesx.h"
-#include "functionsx.h"
-
 #include "nodesx.h"
 #include "segmentsx.h"
 #include "waysx.h"
 #include "relationsx.h"
 
+#include "osmparser.h"
+
 #include "xmlparse.h"
 #include "tagging.h"
 
@@ -90,7 +90,7 @@ static double parse_length(way_t id,const char *k,const char *v);
 /* The XML tag processing function prototypes */
 
 //static int xmlDeclaration_function(const char *_tag_,int _type_,const char *version,const char *encoding);
-//static int osmType_function(const char *_tag_,int _type_);
+static int osmType_function(const char *_tag_,int _type_,const char *version);
 static int relationType_function(const char *_tag_,int _type_,const char *id);
 static int wayType_function(const char *_tag_,int _type_,const char *id);
 static int memberType_function(const char *_tag_,int _type_,const char *type,const char *ref,const char *role);
@@ -162,8 +162,8 @@ static xmltag relationType_tag=
 /*+ The osmType type tag. +*/
 static xmltag osmType_tag=
               {"osm",
-               0, {NULL},
-               NULL,
+               1, {"version"},
+               osmType_function,
                {&boundsType_tag,&boundType_tag,&nodeType_tag,&wayType_tag,&relationType_tag,NULL}};
 
 /*+ The xmlDeclaration type tag. +*/
@@ -525,12 +525,20 @@ static int relationType_function(const char *_tag_,int _type_,const char *id)
   const char *_tag_ Set to the name of the element tag that triggered this function call.
 
   int _type_ Set to XMLPARSE_TAG_START at the start of a tag and/or XMLPARSE_TAG_END at the end of a tag.
+
+  const char *version The contents of the 'version' attribute (or NULL if not defined).
   ++++++++++++++++++++++++++++++++++++++*/
 
-//static int osmType_function(const char *_tag_,int _type_)
-//{
-// return(0);
-//}
+static int osmType_function(const char *_tag_,int _type_,const char *version)
+{
+ if(_type_&XMLPARSE_TAG_START)
+   {
+    if(!version || strcmp(version,"0.6"))
+       XMLPARSE_MESSAGE(_tag_,"Invalid value for 'version' (only '0.6' accepted)");
+   }
+
+ return(0);
+}
 
 
 /*++++++++++++++++++++++++++++++++++++++
@@ -760,7 +768,7 @@ static void process_node_tags(TagList *tags,node_t id,double latitude,double lon
 static void process_way_tags(TagList *tags,way_t id)
 {
  Way   way={0};
- int   oneway=0;
+ int   oneway=0,roundabout=0,area=0;
  char *name=NULL,*ref=NULL,*refname=NULL;
  int i;
 
@@ -794,6 +802,17 @@ static void process_way_tags(TagList *tags,way_t id)
 
     switch(*k)
       {
+      case 'a':
+       if(!strcmp(k,"area"))
+         {
+          if(ISTRUE(v))
+             area=1;
+          else if(!ISFALSE(v))
+             logerror("Way %"Pway_t" has an unrecognised tag value 'area' = '%s' (after tagging rules); using 'no'.\n",id,v);
+         }
+
+       break;
+
       case 'b':
        if(!strcmp(k,"bicycle"))
          {
@@ -965,6 +984,14 @@ static void process_way_tags(TagList *tags,way_t id)
        if(!strcmp(k,"ref"))
           ref=v;
 
+       if(!strcmp(k,"roundabout"))
+         {
+          if(ISTRUE(v))
+             roundabout=1;
+          else if(!ISFALSE(v))
+             logerror("Way %"Pway_t" has an unrecognised tag value 'roundabout' = '%s' (after tagging rules); using 'no'.\n",id,v);
+         }
+
        break;
 
       case 't':
@@ -1009,6 +1036,9 @@ static void process_way_tags(TagList *tags,way_t id)
  if(oneway)
     way.type|=Way_OneWay;
 
+ if(roundabout)
+    way.type|=Way_Roundabout;
+
  if(ref && name)
    {
     refname=(char*)malloc(strlen(ref)+strlen(name)+4);
@@ -1032,11 +1062,11 @@ static void process_way_tags(TagList *tags,way_t id)
     node_t to  =way_nodes[i];
 
     if(oneway>0)
-       AppendSegment(segments,id,from,to,ONEWAY_1TO2);
+       AppendSegment(segments,id,from,to,area+ONEWAY_1TO2);
     else if(oneway<0)
-       AppendSegment(segments,id,from,to,ONEWAY_2TO1);
+       AppendSegment(segments,id,from,to,area+ONEWAY_2TO1);
     else
-       AppendSegment(segments,id,from,to,0);
+       AppendSegment(segments,id,from,to,area);
    }
 }
 
