@@ -1,11 +1,9 @@
 /***************************************
- $Header: /home/amb/routino/src/RCS/planetsplitter.c,v 1.82 2010/11/13 14:22:28 amb Exp $
-
  OSM planet file splitter.
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2010 Andrew M. Bishop
+ This file Copyright 2008-2012 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -36,11 +34,12 @@
 #include "waysx.h"
 #include "relationsx.h"
 #include "superx.h"
+#include "prunex.h"
 
 #include "files.h"
 #include "logging.h"
 #include "functions.h"
-#include "functionsx.h"
+#include "osmparser.h"
 #include "tagging.h"
 
 
@@ -69,10 +68,11 @@ int main(int argc,char** argv)
  WaysX      *Ways;
  RelationsX *Relations;
  int         iteration=0,quit=0;
- int         max_iterations=10;
- char       *dirname=NULL,*prefix=NULL,*tagging=NULL;
+ int         max_iterations=5;
+ char       *dirname=NULL,*prefix=NULL,*tagging=NULL,*errorlog=NULL;
  int         option_parse_only=0,option_process_only=0;
  int         option_filenames=0;
+ int         option_prune_isolated=500,option_prune_short=5,option_prune_straight=3;
  int         arg;
 
  /* Parse the command line arguments */
@@ -95,10 +95,27 @@ int main(int argc,char** argv)
        option_process_only=1;
     else if(!strcmp(argv[arg],"--loggable"))
        option_loggable=1;
+    else if(!strcmp(argv[arg],"--errorlog"))
+       errorlog="error.log";
+    else if(!strncmp(argv[arg],"--errorlog=",11))
+       errorlog=&argv[arg][11];
     else if(!strncmp(argv[arg],"--max-iterations=",17))
        max_iterations=atoi(&argv[arg][17]);
     else if(!strncmp(argv[arg],"--tagging=",10))
        tagging=&argv[arg][10];
+    else if(!strncmp(argv[arg],"--prune",7))
+      {
+       if(!strcmp(&argv[arg][7],"-none"))
+          option_prune_isolated=option_prune_short=option_prune_straight=0;
+       else if(!strncmp(&argv[arg][7],"-isolated=",10))
+          option_prune_isolated=atoi(&argv[arg][17]);
+       else if(!strncmp(&argv[arg][7],"-short=",7))
+          option_prune_short=atoi(&argv[arg][14]);
+       else if(!strncmp(&argv[arg][7],"-straight=",10))
+          option_prune_straight=atoi(&argv[arg][17]);
+       else
+          print_usage(0,argv[arg],NULL);
+      }
     else if(argv[arg][0]=='-' && argv[arg][1]=='-')
        print_usage(0,argv[arg],NULL);
     else
@@ -116,9 +133,9 @@ int main(int argc,char** argv)
  if(!option_filesort_ramsize)
    {
 #if SLIM
-       option_filesort_ramsize=64*1024*1024;
+    option_filesort_ramsize=64*1024*1024;
 #else
-       option_filesort_ramsize=256*1024*1024;
+    option_filesort_ramsize=256*1024*1024;
 #endif
    }
  else
@@ -169,6 +186,11 @@ int main(int argc,char** argv)
 
  Relations=NewRelationList(option_parse_only||option_process_only);
 
+ /* Create the error log file */
+
+ if(errorlog)
+    open_errorlog(FileName(dirname,prefix,errorlog),option_parse_only||option_process_only);
+
  /* Parse the file */
 
  if(option_filenames)
@@ -216,6 +238,8 @@ int main(int argc,char** argv)
     return(0);
    }
 
+ DeleteXMLTaggingRules();
+
  /* Process the data */
 
  printf("\nProcess OSM Data\n================\n\n");
@@ -225,22 +249,12 @@ int main(int argc,char** argv)
 
  SortNodeList(Nodes);
 
- SortSegmentList(Segments);
+ SortSegmentList(Segments,0);
 
  SortWayList(Ways);
 
  SortRelationList(Relations);
 
- /* Process the route relations (must be before compacting the ways) */
-
- ProcessRouteRelations(Relations,Ways);
-
- FreeRelationList(Relations,0);
-
- /* Compact the ways (must be before measuring the segments) */
-
- CompactWayList(Ways);
-
  /* Remove bad segments (must be after sorting the nodes and segments) */
 
  RemoveBadSegments(Nodes,Segments);
@@ -249,15 +263,52 @@ int main(int argc,char** argv)
 
  RemoveNonHighwayNodes(Nodes,Segments);
 
+ /* Process the route relations and first part of turn relations (must be before compacting the ways) */
+
+ ProcessRouteRelations(Relations,Ways);
+
+ ProcessTurnRelations1(Relations,Nodes,Ways);
+
+ /* Compact the ways (must be before measuring the segments) */
+
+ CompactWayList(Ways);
+
  /* Measure the segments and replace node/way id with index (must be after removing non-highway nodes) */
 
- UpdateSegments(Segments,Nodes,Ways);
+ MeasureSegments(Segments,Nodes,Ways);
 
+ /* Index the segments */
+
+ IndexSegments(Segments,Nodes);
+
+ /* Convert the turn relations from ways into nodes */
+
+ ProcessTurnRelations2(Relations,Nodes,Segments,Ways);
+
+ /* Prune unwanted nodes/segments. */
+
+ if(option_prune_straight || option_prune_isolated || option_prune_short)
+   {
+    StartPruning(Nodes,Segments,Ways);
+
+    if(option_prune_straight)
+       PruneStraightHighwayNodes(Nodes,Segments,Ways,option_prune_straight);
+
+    if(option_prune_isolated)
+       PruneIsolatedRegions(Nodes,Segments,Ways,option_prune_isolated);
+
+    if(option_prune_short)
+       PruneShortSegments(Nodes,Segments,Ways,option_prune_short);
+
+    FinishPruning(Nodes,Segments,Ways);
+   }
 
  /* Repeated iteration on Super-Nodes and Super-Segments */
 
  do
    {
+    int nsuper;
+
     printf("\nProcess Super-Data (iteration %d)\n================================%s\n\n",iteration,iteration>9?"=":"");
     fflush(stdout);
 
@@ -269,7 +320,9 @@ int main(int argc,char** argv)
 
        /* Select the super-segments */
 
-       SuperSegments=CreateSuperSegments(Nodes,Segments,Ways,iteration);
+       SuperSegments=CreateSuperSegments(Nodes,Segments,Ways);
+
+       nsuper=Segments->number;
       }
     else
       {
@@ -281,10 +334,9 @@ int main(int argc,char** argv)
 
        /* Select the super-segments */
 
-       SuperSegments2=CreateSuperSegments(Nodes,SuperSegments,Ways,iteration);
+       SuperSegments2=CreateSuperSegments(Nodes,SuperSegments,Ways);
 
-       if(SuperSegments->xnumber==SuperSegments2->xnumber)
-          quit=1;
+       nsuper=SuperSegments->number;
 
        FreeSegmentList(SuperSegments,0);
 
@@ -293,12 +345,21 @@ int main(int argc,char** argv)
 
     /* Sort the super-segments */
 
-    SortSegmentList(SuperSegments);
+    SortSegmentList(SuperSegments,0);
 
     /* Remove duplicated super-segments */
 
     DeduplicateSegments(SuperSegments,Nodes,Ways);
 
+    /* Index the segments */
+
+    IndexSegments(SuperSegments,Nodes);
+
+    /* Check for end condition */
+
+    if(SuperSegments->number==nsuper)
+       quit=1;
+
     iteration++;
 
     if(iteration>max_iterations)
@@ -321,39 +382,39 @@ int main(int argc,char** argv)
 
  Segments=MergedSegments;
 
- /* Rotate segments so that node1<node2 */
+ /* Sort and re-index the segments */
 
- RotateSegments(Segments);
+ SortSegmentList(Segments,0);
 
- /* Sort the segments */
-
- SortSegmentList(Segments);
-
- /* Remove duplicated segments */
-
- DeduplicateSegments(Segments,Nodes,Ways);
+ IndexSegments(Segments,Nodes);
 
  /* Cross reference the nodes and segments */
 
  printf("\nCross-Reference Nodes and Segments\n==================================\n\n");
  fflush(stdout);
 
- /* Sort the node list geographically */
+ /* Sort the nodes geographically and update the segment indexes accordingly */
 
  SortNodeListGeographically(Nodes);
 
- /* Create the real segments and nodes */
+ UpdateSegments(Segments,Nodes,Ways);
 
- CreateRealNodes(Nodes,iteration);
+ /* Sort the segments geographically and re-index them */
 
- CreateRealSegments(Segments,Ways);
-
- /* Fix the segment and node indexes */
-
- IndexNodes(Nodes,Segments);
+ SortSegmentList(Segments,0);
 
  IndexSegments(Segments,Nodes);
 
+ /* Update the nodes */
+
+ UpdateNodes(Nodes,Segments);
+
+ /* Fix the turn relations after sorting nodes geographically */
+
+ UpdateTurnRelations(Relations,Nodes,Segments);
+
+ SortTurnRelationList(Relations);
+
  /* Output the results */
 
  printf("\nWrite Out Database Files\n========================\n\n");
@@ -377,6 +438,17 @@ int main(int argc,char** argv)
 
  FreeWayList(Ways,0);
 
+ /* Write out the relations */
+
+ SaveRelationList(Relations,FileName(dirname,prefix,"relations.mem"));
+
+ FreeRelationList(Relations,0);
+
+ /* Close the error log file */
+
+ if(errorlog)
+    close_errorlog();
+
  return(0);
 }
 
@@ -398,10 +470,14 @@ static void print_usage(int detail,const char *argerr,const char *err)
          "                      [--dir=<dirname>] [--prefix=<name>]\n"
          "                      [--sort-ram-size=<size>]\n"
          "                      [--tmpdir=<dirname>]\n"
-         "                      [--parse-only | --process-only]\n"
-         "                      [--loggable]\n"
-         "                      [--max-iterations=<number>]\n"
          "                      [--tagging=<filename>]\n"
+         "                      [--loggable] [--errorlog[=<name>]]\n"
+         "                      [--parse-only | --process-only]\n"
+         "                      [--max-iterations=<number>]\n"
+         "                      [--prune-none]\n"
+         "                      [--prune-isolated=<len>]\n"
+         "                      [--prune-short=<len>]\n"
+         "                      [--prune-straight=<len>]\n"
          "                      [<filename.osm> ...]\n");
 
  if(argerr)
@@ -431,18 +507,30 @@ static void print_usage(int detail,const char *argerr,const char *err)
             "--tmpdir=<dirname>        The directory name for temporary files.\n"
             "                          (defaults to the '--dir' option directory.)\n"
             "\n"
-            "--parse-only              Parse the input OSM files and store the results.\n"
-            "--process-only            Process the stored results from previous option.\n"
-            "\n"
-            "--loggable                Print progress messages suitable for logging to file.\n"
-            "\n"
-            "--max-iterations=<number> The number of iterations for finding super-nodes.\n"
-            "\n"
             "--tagging=<filename>      The name of the XML file containing the tagging rules\n"
             "                          (defaults to 'tagging.xml' with '--dir' and\n"
             "                           '--prefix' options or the file installed in\n"
             "                           '" DATADIR "').\n"
             "\n"
+            "--loggable                Print progress messages suitable for logging to file.\n"
+            "--errorlog[=<name>]       Log parsing errors to 'error.log' or the given name\n"
+            "                          (the '--dir' and '--prefix' options are applied).\n"
+            "\n"
+            "--parse-only              Parse the input OSM files and store the results.\n"
+            "--process-only            Process the stored results from previous option.\n"
+            "\n"
+            "--max-iterations=<number> The number of iterations for finding super-nodes\n"
+            "                          (defaults to 5).\n"
+            "\n"
+            "--prune-none              Disable the prune options below, they are re-enabled\n"
+            "                          by adding them to the command line after this option.\n"
+            "--prune-isolated=<len>    Remove small disconnected groups of segments\n"
+            "                          (defaults to removing groups under 500m).\n"
+            "--prune-short=<len>       Remove short segments (defaults to removing segments\n"
+            "                          up to a maximum length of 5m).\n"
+            "--prune-straight=<len>    Remove nodes in almost straight highways (defaults to\n"
+            "                          removing nodes up to 3m offset from a straight line).\n"
+            "\n"
             "<filename.osm> ...        The name(s) of the file(s) to process (by default\n"
             "                          data is read from standard input).\n"
             "\n"
