@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2010-2011 Andrew M. Bishop
+ This file Copyright 2010-2012 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -22,11 +22,10 @@
 
 #include <assert.h>
 #include <stdlib.h>
-#include <stdio.h>
 #include <string.h>
-#include <sys/stat.h>
 
 #include "types.h"
+#include "segments.h"
 #include "relations.h"
 
 #include "nodesx.h"
@@ -52,8 +51,6 @@ static int sort_by_via(TurnRestrictRelX *a,TurnRestrictRelX *b);
 /*+ The command line '--tmpdir' option or its default value. +*/
 extern char *option_tmpdirname;
 
-static RelationsX* sortrelationsx;
-
 
 /*++++++++++++++++++++++++++++++++++++++
   Allocate a new relation list (create a new file or open an existing one).
@@ -230,7 +227,7 @@ void AppendTurnRestrictRelation(RelationsX* relationsx,relation_t id,
                                 way_t from,way_t to,node_t via,
                                 TurnRestriction restriction,transports_t except)
 {
- TurnRestrictRelX relationx;
+ TurnRestrictRelX relationx={0};
 
  relationx.id=id;
  relationx.from=from;
@@ -289,9 +286,7 @@ void SortRelationList(RelationsX* relationsx)
     trxnumber=relationsx->trnumber;
     relationsx->trnumber=0;
 
-    sortrelationsx=relationsx;
-
-    filesort_fixed(relationsx->trfd,trfd,sizeof(TurnRestrictRelX),(int (*)(const void*,const void*))sort_by_id,(int (*)(void*,index_t))deduplicate_by_id);
+    relationsx->trnumber=filesort_fixed(relationsx->trfd,trfd,sizeof(TurnRestrictRelX),(int (*)(const void*,const void*))sort_by_id,(int (*)(void*,index_t))deduplicate_by_id);
 
     /* Close the files */
 
@@ -347,8 +342,6 @@ static int deduplicate_by_id(TurnRestrictRelX *relationx,index_t index)
    {
     previd=relationx->id;
 
-    sortrelationsx->trnumber++;
-
     return(1);
    }
  else
@@ -548,7 +541,7 @@ void ProcessRouteRelations(RelationsX *relationsx,WaysX *waysx)
                 if(routes&Transports_Bicycle)
                    wayx->way.props|=Properties_BicycleRoute;
 
-                PutBackWayX(waysx,way,1);
+                PutBackWayX(waysx,wayx);
 
                 ways++;
                }
@@ -802,7 +795,7 @@ void ProcessTurnRelations2(RelationsX *relationsx,NodesX *nodesx,SegmentsX *segm
                 vehicles_to=0;  /* not allowed */
             }
 
-          segmentx=NextSegmentX(segmentsx,segmentx,relationx.via,1);
+          segmentx=NextSegmentX(segmentsx,segmentx,relationx.via);
          }
 
        if(node_from==NO_NODE)
@@ -901,7 +894,7 @@ void ProcessTurnRelations2(RelationsX *relationsx,NodesX *nodesx,SegmentsX *segm
                }
             }
 
-          segmentx=NextSegmentX(segmentsx,segmentx,relationx.via,1);
+          segmentx=NextSegmentX(segmentsx,segmentx,relationx.via);
          }
 
        if(node_from==NO_NODE)
@@ -945,7 +938,7 @@ void ProcessTurnRelations2(RelationsX *relationsx,NodesX *nodesx,SegmentsX *segm
 
     nodex=LookupNodeX(nodesx,relationx.via,1);
     nodex->flags|=NODE_TURNRSTRCT;
-    PutBackNodeX(nodesx,relationx.via,1);
+    PutBackNodeX(nodesx,nodex);
 
     segmentx=FirstSegmentX(segmentsx,relationx.via,1);
 
@@ -955,9 +948,9 @@ void ProcessTurnRelations2(RelationsX *relationsx,NodesX *nodesx,SegmentsX *segm
 
        nodex=LookupNodeX(nodesx,othernode,1);
        nodex->flags|=NODE_TURNRSTRCT2;
-       PutBackNodeX(nodesx,othernode,1);
+       PutBackNodeX(nodesx,nodex);
 
-       segmentx=NextSegmentX(segmentsx,segmentx,relationx.via,1);
+       segmentx=NextSegmentX(segmentsx,segmentx,relationx.via);
       }
 
    endloop: ;
@@ -1001,11 +994,11 @@ void ProcessTurnRelations2(RelationsX *relationsx,NodesX *nodesx,SegmentsX *segm
 void UpdateTurnRelations(RelationsX *relationsx,NodesX *nodesx,SegmentsX *segmentsx)
 {
  int trfd;
- index_t i;
+ index_t i,kept=0;
 
  /* Print the start message */
 
- printf_first("Updating Turn Relations: Relations=0");
+ printf_first("Updating Turn Relations (Deleting Pruned): Relations=0");
 
  /* Map into memory / open the files */
 
@@ -1028,7 +1021,6 @@ void UpdateTurnRelations(RelationsX *relationsx,NodesX *nodesx,SegmentsX *segmen
  for(i=0;i<relationsx->trnumber;i++)
    {
     TurnRestrictRelX relationx;
-    SegmentX *segmentx;
     index_t from_node,via_node,to_node;
 
     ReadFile(relationsx->trfd,&relationx,sizeof(TurnRestrictRelX));
@@ -1037,26 +1029,31 @@ void UpdateTurnRelations(RelationsX *relationsx,NodesX *nodesx,SegmentsX *segmen
     via_node =nodesx->gdata[relationx.via];
     to_node  =nodesx->gdata[relationx.to];
 
-    segmentx=FirstSegmentX(segmentsx,via_node,1);
-
-    do
+    if(from_node<nodesx->number && via_node<nodesx->number && to_node<nodesx->number)
       {
-       if(OtherNode(segmentx,via_node)==from_node)
-          relationx.from=IndexSegmentX(segmentsx,segmentx);
+       SegmentX *segmentx=FirstSegmentX(segmentsx,via_node,1);
 
-       if(OtherNode(segmentx,via_node)==to_node)
-          relationx.to=IndexSegmentX(segmentsx,segmentx);
+       do
+         {
+          if(OtherNode(segmentx,via_node)==from_node)
+             relationx.from=IndexSegmentX(segmentsx,segmentx);
 
-       segmentx=NextSegmentX(segmentsx,segmentx,via_node,1);
+          if(OtherNode(segmentx,via_node)==to_node)
+             relationx.to=IndexSegmentX(segmentsx,segmentx);
+
+          segmentx=NextSegmentX(segmentsx,segmentx,via_node);
+         }
+       while(segmentx);
+
+       relationx.via=via_node;
+
+       WriteFile(trfd,&relationx,sizeof(TurnRestrictRelX));
+
+       kept++;
       }
-    while(segmentx);
-
-    relationx.via=via_node;
-
-    WriteFile(trfd,&relationx,sizeof(TurnRestrictRelX));
 
     if(!(relationsx->trnumber%1000))
-       printf_middle("Updating Turn Relations: Relations=%"Pindex_t,relationsx->trnumber);
+       printf_middle("Updating Turn Relations (Deleting Pruned): Relations=%"Pindex_t,relationsx->trnumber);
    }
 
  /* Close the files */
@@ -1074,7 +1071,9 @@ void UpdateTurnRelations(RelationsX *relationsx,NodesX *nodesx,SegmentsX *segmen
 
  /* Print the final message */
 
- printf_last("Updated Turn Relations: Relations=%"Pindex_t,relationsx->trnumber);
+ printf_last("Updated Turn Relations: Relations=%"Pindex_t" Deleted=%"Pindex_t,kept,relationsx->trnumber-kept);
+
+ relationsx->trnumber=kept;
 }
 
 
@@ -1109,7 +1108,7 @@ void SaveRelationList(RelationsX* relationsx,const char *filename)
  for(i=0;i<relationsx->trnumber;i++)
    {
     TurnRestrictRelX relationx;
-    TurnRelation relation;
+    TurnRelation relation={0};
 
     ReadFile(relationsx->trfd,&relationx,sizeof(TurnRestrictRelX));
 
