@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2011 Andrew M. Bishop
+ This file Copyright 2008-2012 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -23,21 +23,17 @@
 #include <assert.h>
 #include <math.h>
 #include <stdlib.h>
-#include <stdio.h>
 #include <string.h>
-#include <sys/stat.h>
 
 #include "types.h"
-#include "nodes.h"
 #include "segments.h"
 #include "ways.h"
 
+#include "typesx.h"
 #include "nodesx.h"
 #include "segmentsx.h"
 #include "waysx.h"
 
-#include "types.h"
-
 #include "files.h"
 #include "logging.h"
 #include "sorting.h"
@@ -51,6 +47,7 @@ extern char *option_tmpdirname;
 /* Local functions */
 
 static int sort_by_id(SegmentX *a,SegmentX *b);
+static int delete_pruned(SegmentX *segmentx,index_t index);
 
 static distance_t DistanceX(NodeX *nodex1,NodeX *nodex2);
 
@@ -110,12 +107,15 @@ void FreeSegmentList(SegmentsX *segmentsx,int keep)
 
  free(segmentsx->filename);
 
- if(segmentsx->usednode)
-    free(segmentsx->usednode);
-
  if(segmentsx->firstnode)
     free(segmentsx->firstnode);
 
+ if(segmentsx->next1)
+    free(segmentsx->next1);
+
+ if(segmentsx->usednode)
+    free(segmentsx->usednode);
+
  free(segmentsx);
 }
 
@@ -167,16 +167,22 @@ void AppendSegment(SegmentsX *segmentsx,way_t way,node_t node1,node_t node2,dist
 /*++++++++++++++++++++++++++++++++++++++
   Sort the segment list.
 
-  SegmentsX *segmentsx The set of segments to modify.
+  SegmentsX *segmentsx The set of segments to sort and modify.
+
+  int delete Set to true if pruned segments are to be deleted.
   ++++++++++++++++++++++++++++++++++++++*/
 
-void SortSegmentList(SegmentsX *segmentsx)
+void SortSegmentList(SegmentsX *segmentsx,int delete)
 {
  int fd;
+ index_t kept;
 
  /* Print the start message */
 
- printf_first("Sorting Segments");
+ if(delete)
+    printf_first("Sorting Segments (Deleting Pruned)");
+ else
+    printf_first("Sorting Segments");
 
  /* Close the file (finished appending) */
 
@@ -193,7 +199,10 @@ void SortSegmentList(SegmentsX *segmentsx)
 
  /* Sort by node indexes */
 
- filesort_fixed(segmentsx->fd,fd,sizeof(SegmentX),(int (*)(const void*,const void*))sort_by_id,NULL);
+ if(delete)
+    kept=filesort_fixed(segmentsx->fd,fd,sizeof(SegmentX),(int (*)(const void*,const void*))sort_by_id,(int (*)(void*,index_t))delete_pruned);
+ else
+    filesort_fixed(segmentsx->fd,fd,sizeof(SegmentX),(int (*)(const void*,const void*))sort_by_id,NULL);
 
  /* Close the files */
 
@@ -202,7 +211,13 @@ void SortSegmentList(SegmentsX *segmentsx)
 
  /* Print the final message */
 
- printf_last("Sorted Segments: Segments=%"Pindex_t,segmentsx->number);
+ if(delete)
+   {
+    printf_last("Sorted Segments: Segments=%"Pindex_t" Deleted=%"Pindex_t,kept,segmentsx->number-kept);
+    segmentsx->number=kept;
+   }
+ else
+    printf_last("Sorted Segments: Segments=%"Pindex_t,segmentsx->number);
 }
 
 
@@ -236,8 +251,8 @@ static int sort_by_id(SegmentX *a,SegmentX *b)
        return(1);
     else
       {
-       distance_t a_distance=a->distance;
-       distance_t b_distance=b->distance;
+       distance_t a_distance=DISTANCE(a->distance);
+       distance_t b_distance=DISTANCE(b->distance);
 
        if(a_distance<b_distance)
           return(-1);
@@ -250,6 +265,25 @@ static int sort_by_id(SegmentX *a,SegmentX *b)
 }
 
 
+/*++++++++++++++++++++++++++++++++++++++
+  Delete the pruned segments.
+
+  int delete_pruned Return 1 if the value is to be kept, otherwise 0.
+
+  SegmentX *segmentx The extended segment.
+
+  index_t index The index of this segment in the total.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static int delete_pruned(SegmentX *segmentx,index_t index)
+{
+ if(IsPrunedSegmentX(segmentx))
+    return(0);
+
+ return(1);
+}
+
+
 /*++++++++++++++++++++++++++++++++++++++
   Find the first extended segment with a particular starting node index.
  
@@ -272,9 +306,6 @@ SegmentX *FirstSegmentX(SegmentsX *segmentsx,index_t nodeindex,int position)
 
  segmentx=LookupSegmentX(segmentsx,index,position);
 
- if(segmentx->node1!=nodeindex && segmentx->node2!=nodeindex)
-    return(NULL);
-
  return(segmentx);
 }
 
@@ -289,39 +320,56 @@ SegmentX *FirstSegmentX(SegmentsX *segmentsx,index_t nodeindex,int position)
   SegmentX *segmentx The current segment.
 
   index_t nodeindex The node index.
-
-  int position A flag to pass through.
   ++++++++++++++++++++++++++++++++++++++*/
 
-SegmentX *NextSegmentX(SegmentsX *segmentsx,SegmentX *segmentx,index_t nodeindex,int position)
+SegmentX *NextSegmentX(SegmentsX *segmentsx,SegmentX *segmentx,index_t nodeindex)
 {
+#if SLIM
+ int position=1+(segmentx-&segmentsx->cached[0]);
+#endif
+
  if(segmentx->node1==nodeindex)
    {
-#if SLIM
-    index_t index=IndexSegmentX(segmentsx,segmentx);
-    index++;
+    if(segmentsx->next1)
+      {
+       index_t index=IndexSegmentX(segmentsx,segmentx);
+
+       if(segmentsx->next1[index]==NO_SEGMENT)
+          return(NULL);
+
+       segmentx=LookupSegmentX(segmentsx,segmentsx->next1[index],position);
 
-    if(index>=segmentsx->number)
-       return(NULL);
-    segmentx=LookupSegmentX(segmentsx,index,position);
-    if(segmentx->node1!=nodeindex)
-       return(NULL);
-    else
        return(segmentx);
+      }
+    else
+      {
+#if SLIM
+       index_t index=IndexSegmentX(segmentsx,segmentx);
+       index++;
+
+       if(index>=segmentsx->number)
+          return(NULL);
+
+       segmentx=LookupSegmentX(segmentsx,index,position);
 #else
-    segmentx++;
-    if(IndexSegmentX(segmentsx,segmentx)>=segmentsx->number || segmentx->node1!=nodeindex)
-       return(NULL);
-    else
-       return(segmentx);
+       segmentx++;
+
+       if(IndexSegmentX(segmentsx,segmentx)>=segmentsx->number)
+          return(NULL);
 #endif
+
+       if(segmentx->node1!=nodeindex)
+          return(NULL);
+
+       return(segmentx);
+      }
    }
  else
    {
     if(segmentx->next2==NO_SEGMENT)
        return(NULL);
-    else
-       return(LookupSegmentX(segmentsx,segmentx->next2,position));
+
+    return(LookupSegmentX(segmentsx,segmentx->next2,position));
    }
 }
  
@@ -340,6 +388,7 @@ void RemoveBadSegments(NodesX *nodesx,SegmentsX *segmentsx)
  SegmentX segmentx;
  int fd;
  node_t prevnode1=NO_NODE_ID,prevnode2=NO_NODE_ID;
+ distance_t prevdist=0;
 
  /* Print the start message */
 
@@ -347,9 +396,9 @@ void RemoveBadSegments(NodesX *nodesx,SegmentsX *segmentsx)
 
  /* Allocate the array of node flags */
 
- segmentsx->usednode=(char*)calloc((1+nodesx->number/8),sizeof(char));
+ segmentsx->usednode=AllocBitMask(nodesx->number);
 
- assert(segmentsx->usednode); /* Check malloc() worked */
+ assert(segmentsx->usednode); /* Check AllocBitMask() worked */
 
  /* Re-open the file read-only and a new file writeable */
 
@@ -368,7 +417,17 @@ void RemoveBadSegments(NodesX *nodesx,SegmentsX *segmentsx)
 
     if(prevnode1==segmentx.node1 && prevnode2==segmentx.node2)
       {
-       logerror("Segment connecting nodes %"Pnode_t" and %"Pnode_t" is duplicated.\n",segmentx.node1,segmentx.node2);
+       if(!prevdist && !DISTANCE(segmentx.distance))
+          logerror("Segment connecting nodes %"Pnode_t" and %"Pnode_t" is duplicated.\n",segmentx.node1,segmentx.node2);
+
+       if(!prevdist && DISTANCE(segmentx.distance))
+          logerror("Segment connecting nodes %"Pnode_t" and %"Pnode_t" is duplicated (discarded the area).\n",segmentx.node1,segmentx.node2);
+
+       if(prevdist && !DISTANCE(segmentx.distance))
+          logerror("Segment connecting nodes %"Pnode_t" and %"Pnode_t" is duplicated (discarded the non-area).\n",segmentx.node1,segmentx.node2);
+
+       if(prevdist && DISTANCE(segmentx.distance))
+          logerror("Segment connecting nodes %"Pnode_t" and %"Pnode_t" is duplicated (both are areas).\n",segmentx.node1,segmentx.node2);
 
        duplicate++;
       }
@@ -402,6 +461,7 @@ void RemoveBadSegments(NodesX *nodesx,SegmentsX *segmentsx)
 
        prevnode1=segmentx.node1;
        prevnode2=segmentx.node2;
+       prevdist=DISTANCE(segmentx.distance);
       }
 
     total++;
@@ -476,7 +536,7 @@ void MeasureSegments(SegmentsX *segmentsx,NodesX *nodesx,WaysX *waysx)
     segmentx.node2=node2;
     segmentx.way  =way;
 
-    /* Set the distance but preserve the ONEWAY_* flags */
+    /* Set the distance but preserve the other flags */
 
     segmentx.distance|=DISTANCE(DistanceX(nodex1,nodex2));
 
@@ -689,7 +749,7 @@ void IndexSegments(SegmentsX *segmentsx,NodesX *nodesx)
 
     segmentx->next2=segmentsx->firstnode[segmentx->node2];
 
-    PutBackSegmentX(segmentsx,index,1);
+    PutBackSegmentX(segmentsx,segmentx);
 
     segmentsx->firstnode[segmentx->node1]=index;
     segmentsx->firstnode[segmentx->node2]=index;
@@ -832,7 +892,7 @@ void SaveSegmentList(SegmentsX *segmentsx,const char *filename)
  for(i=0;i<segmentsx->number;i++)
    {
     SegmentX segmentx;
-    Segment  segment;
+    Segment  segment={0};
 
     ReadFile(segmentsx->fd,&segmentx,sizeof(SegmentX));
 
