@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2011 Andrew M. Bishop
+ This file Copyright 2008-2012 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -22,10 +22,12 @@
 
 #include <assert.h>
 #include <stdlib.h>
-#include <stdio.h>
 
+#include "types.h"
+#include "segments.h"
 #include "ways.h"
 
+#include "typesx.h"
 #include "nodesx.h"
 #include "segmentsx.h"
 #include "waysx.h"
@@ -63,6 +65,17 @@ void ChooseSuperNodes(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx)
 
  printf_first("Finding Super-Nodes: Nodes=0 Super-Nodes=0");
 
+ /* Allocate and set the super-node markers */
+
+ if(!nodesx->super)
+   {
+    nodesx->super=AllocBitMask(nodesx->number);
+
+    assert(nodesx->super); /* Check AllocBitMask() worked */
+
+    SetAllBits1(nodesx->super,nodesx->number);
+   }
+
  /* Map into memory / open the files */
 
 #if !SLIM
@@ -84,7 +97,9 @@ void ChooseSuperNodes(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx)
        int issuper=0;
        NodeX *nodex=LookupNodeX(nodesx,i,1);
 
-       if(nodex->flags&(NODE_TURNRSTRCT|NODE_TURNRSTRCT2))
+       if(IsPrunedNodeX(nodex))
+          issuper=0;
+       else if(nodex->flags&(NODE_TURNRSTRCT|NODE_TURNRSTRCT2))
           issuper=1;
        else
          {
@@ -143,7 +158,7 @@ void ChooseSuperNodes(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx)
              if(issuper)
                 break;
 
-             segmentx=NextSegmentX(segmentsx,segmentx,i,1);
+             segmentx=NextSegmentX(segmentsx,segmentx,i);
 
              count++;
             }
@@ -277,7 +292,7 @@ SegmentsX *CreateSuperSegments(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx)
              FreeResultsList(results);
             }
 
-          segmentx=NextSegmentX(segmentsx,segmentx,i,1);
+          segmentx=NextSegmentX(segmentsx,segmentx,i);
          }
 
        sn++;
@@ -505,7 +520,7 @@ static Results *FindRoutesWay(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,n
 
       endloop:
 
-       segmentx=NextSegmentX(segmentsx,segmentx,node1,2);
+       segmentx=NextSegmentX(segmentsx,segmentx,node1);
       }
    }
 
