@@ -3,7 +3,7 @@
 //
 // Part of the Routino routing software.
 //
-// This file Copyright 2008-2010 Andrew M. Bishop
+// This file Copyright 2008-2011 Andrew M. Bishop
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU Affero General Public License as published by
@@ -19,81 +19,6 @@
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 //
 
-////////////////////////////////////////////////////////////////////////////////
-/////////////////////////// Routino default profile ////////////////////////////
-////////////////////////////////////////////////////////////////////////////////
-
-var routino={ // contains all default Routino options (generated using "--help-profile-json").
-
-  // Default transport type
-  transport: 'motorcar',
-
-  // Transport types
-  transports: { foot: 1, horse: 2, wheelchair: 3, bicycle: 4, moped: 5, motorbike: 6, motorcar: 7, goods: 8, hgv: 9, psv: 10 },
-
-  // Highway types
-  highways: { motorway: 1, trunk: 2, primary: 3, secondary: 4, tertiary: 5, unclassified: 6, residential: 7, service: 8, track: 9, cycleway: 10, path: 11, steps: 12, ferry: 13 },
-
-  // Property types
-  properties: { paved: 1, multilane: 2, bridge: 3, tunnel: 4, footroute: 5, bicycleroute: 6 },
-
-  // Restriction types
-  restrictions: { oneway: 1, weight: 2, height: 3, width: 4, length: 5 },
-
-  // Allowed highways
-  profile_highway: {
-        motorway: { foot:   0, horse:   0, wheelchair:   0, bicycle:   0, moped:   0, motorbike: 100, motorcar: 100, goods: 100, hgv: 100, psv: 100 },
-           trunk: { foot:  40, horse:  25, wheelchair:  40, bicycle:  30, moped:  90, motorbike: 100, motorcar: 100, goods: 100, hgv: 100, psv: 100 },
-         primary: { foot:  50, horse:  50, wheelchair:  50, bicycle:  70, moped: 100, motorbike:  90, motorcar:  90, goods:  90, hgv:  90, psv:  90 },
-       secondary: { foot:  60, horse:  50, wheelchair:  60, bicycle:  80, moped:  90, motorbike:  80, motorcar:  80, goods:  80, hgv:  80, psv:  80 },
-        tertiary: { foot:  70, horse:  75, wheelchair:  70, bicycle:  90, moped:  80, motorbike:  70, motorcar:  70, goods:  70, hgv:  70, psv:  70 },
-    unclassified: { foot:  80, horse:  75, wheelchair:  80, bicycle:  90, moped:  70, motorbike:  60, motorcar:  60, goods:  60, hgv:  60, psv:  60 },
-     residential: { foot:  90, horse:  75, wheelchair:  90, bicycle:  90, moped:  60, motorbike:  50, motorcar:  50, goods:  50, hgv:  50, psv:  50 },
-         service: { foot:  90, horse:  75, wheelchair:  90, bicycle:  90, moped:  80, motorbike:  80, motorcar:  80, goods:  80, hgv:  80, psv:  80 },
-           track: { foot:  95, horse: 100, wheelchair:  95, bicycle:  90, moped:   0, motorbike:   0, motorcar:   0, goods:   0, hgv:   0, psv:   0 },
-        cycleway: { foot:  95, horse:  90, wheelchair:  95, bicycle: 100, moped:   0, motorbike:   0, motorcar:   0, goods:   0, hgv:   0, psv:   0 },
-            path: { foot: 100, horse: 100, wheelchair: 100, bicycle:  90, moped:   0, motorbike:   0, motorcar:   0, goods:   0, hgv:   0, psv:   0 },
-           steps: { foot:  80, horse:   0, wheelchair:   0, bicycle:   0, moped:   0, motorbike:   0, motorcar:   0, goods:   0, hgv:   0, psv:   0 },
-           ferry: { foot:  20, horse:  20, wheelchair:  20, bicycle:  20, moped:  20, motorbike:  20, motorcar:  20, goods:  20, hgv:  20, psv:  20 }
-     },
-
-  // Speed limits
-  profile_speed: {
-        motorway: { foot:   0, horse:   0, wheelchair:   0, bicycle:   0, moped:  48, motorbike: 112, motorcar: 112, goods:  96, hgv:  89, psv:  89 },
-           trunk: { foot:   4, horse:   8, wheelchair:   4, bicycle:  20, moped:  48, motorbike:  96, motorcar:  96, goods:  96, hgv:  80, psv:  80 },
-         primary: { foot:   4, horse:   8, wheelchair:   4, bicycle:  20, moped:  48, motorbike:  96, motorcar:  96, goods:  96, hgv:  80, psv:  80 },
-       secondary: { foot:   4, horse:   8, wheelchair:   4, bicycle:  20, moped:  48, motorbike:  88, motorcar:  88, goods:  88, hgv:  80, psv:  80 },
-        tertiary: { foot:   4, horse:   8, wheelchair:   4, bicycle:  20, moped:  48, motorbike:  80, motorcar:  80, goods:  80, hgv:  80, psv:  80 },
-    unclassified: { foot:   4, horse:   8, wheelchair:   4, bicycle:  20, moped:  48, motorbike:  64, motorcar:  64, goods:  64, hgv:  64, psv:  64 },
-     residential: { foot:   4, horse:   8, wheelchair:   4, bicycle:  20, moped:  48, motorbike:  48, motorcar:  48, goods:  48, hgv:  48, psv:  48 },
-         service: { foot:   4, horse:   8, wheelchair:   4, bicycle:  20, moped:  32, motorbike:  32, motorcar:  32, goods:  32, hgv:  32, psv:  32 },
-           track: { foot:   4, horse:   8, wheelchair:   4, bicycle:  20, moped:  16, motorbike:  16, motorcar:  16, goods:  16, hgv:  16, psv:  16 },
-        cycleway: { foot:   4, horse:   8, wheelchair:   4, bicycle:  20, moped:   0, motorbike:   0, motorcar:   0, goods:   0, hgv:   0, psv:   0 },
-            path: { foot:   4, horse:   8, wheelchair:   4, bicycle:  20, moped:   0, motorbike:   0, motorcar:   0, goods:   0, hgv:   0, psv:   0 },
-           steps: { foot:   4, horse:   0, wheelchair:   4, bicycle:   0, moped:   0, motorbike:   0, motorcar:   0, goods:   0, hgv:   0, psv:   0 },
-           ferry: { foot:  10, horse:  10, wheelchair:  10, bicycle:  10, moped:  10, motorbike:  10, motorcar:  10, goods:  10, hgv:  10, psv:  10 }
-     },
-
-  // Highway properties
-  profile_property: {
-           paved: { foot:  50, horse:  20, wheelchair:  90, bicycle:  50, moped: 100, motorbike: 100, motorcar: 100, goods: 100, hgv: 100, psv: 100 },
-       multilane: { foot:  25, horse:  25, wheelchair:  25, bicycle:  25, moped:  35, motorbike:  60, motorcar:  60, goods:  60, hgv:  60, psv:  60 },
-          bridge: { foot:  50, horse:  50, wheelchair:  50, bicycle:  50, moped:  50, motorbike:  50, motorcar:  50, goods:  50, hgv:  50, psv:  50 },
-          tunnel: { foot:  50, horse:  50, wheelchair:  50, bicycle:  50, moped:  50, motorbike:  50, motorcar:  50, goods:  50, hgv:  50, psv:  50 },
-       footroute: { foot:  55, horse:  50, wheelchair:  55, bicycle:  50, moped:  50, motorbike:  50, motorcar:  45, goods:  45, hgv:  45, psv:  45 },
-    bicycleroute: { foot:  55, horse:  50, wheelchair:  55, bicycle:  60, moped:  50, motorbike:  50, motorcar:  45, goods:  45, hgv:  45, psv:  45 }
-     },
-
-  // Restrictions
-  profile_restrictions: {
-          oneway: { foot:    0, horse:    1, wheelchair:    0, bicycle:    1, moped:    1, motorbike:    1, motorcar:    1, goods:    1, hgv:    1, psv:    1 },
-          weight: { foot:  0.0, horse:  0.0, wheelchair:  0.0, bicycle:  0.0, moped:  0.0, motorbike:  0.0, motorcar:  0.0, goods:  5.0, hgv: 10.0, psv: 15.0 },
-          height: { foot:  0.0, horse:  0.0, wheelchair:  0.0, bicycle:  0.0, moped:  0.0, motorbike:  0.0, motorcar:  0.0, goods:  2.5, hgv:  3.0, psv:  3.0 },
-           width: { foot:  0.0, horse:  0.0, wheelchair:  0.0, bicycle:  0.0, moped:  0.0, motorbike:  0.0, motorcar:  0.0, goods:  2.0, hgv:  2.5, psv:  2.5 },
-          length: { foot:  0.0, horse:  0.0, wheelchair:  0.0, bicycle:  0.0, moped:  0.0, motorbike:  0.0, motorcar:  0.0, goods:  5.0, hgv:  6.0, psv:  6.0 }
-     }
-
-}; // end of routino variable
 
 // Make a deep copy of the routino profile.
 
@@ -170,7 +95,7 @@ function form_init()
 
     for(var key in routino.restrictions)
       {
-       if(key=="oneway")
+       if(key=="oneway" || key=="turns")
           formSetRestriction(key);
        else
          {
@@ -273,7 +198,7 @@ function formSetTransport(type)
 
  for(var key in routino.restrictions)
    {
-    if(key=="oneway")
+    if(key=="oneway" || key=="turns")
        document.forms["form"].elements["restrict-" + key].checked=routino.profile_restrictions[key][routino.transport];
     else
        document.forms["form"].elements["restrict-" + key].value=routino.profile_restrictions[key][routino.transport];
@@ -328,12 +253,12 @@ function formSetProperty(type)
 
 
 //
-// Change of oneway rule in the form
+// Change of Restriction rule in the form
 //
 
 function formSetRestriction(type)
 {
- if(type=="oneway")
+ if(type=="oneway" || type=="turns")
     routino.profile_restrictions[type][routino.transport]=document.forms["form"].elements["restrict-" + type].checked;
  else
     routino.profile_restrictions[type][routino.transport]=document.forms["form"].elements["restrict-" + type].value;
@@ -516,7 +441,7 @@ function discardReturnKey(ev)
 ////////////////////////////////////////////////////////////////////////////////
 
 var map;
-var layerMapOSM, layerVectors, layerGPX;
+var layerMap=[], layerVectors, layerGPX;
 var epsg4326, epsg900913;
 var map_args;
 
@@ -526,21 +451,8 @@ var map_args;
 
 function map_init(lat,lon,zoom)
 {
- // Default configuration:
- // UK coordinate range
- // West -11.0, South 49.5, East 2.0, North 61.0
- // Zoom level 4 to 15
-
- // EDIT THIS below to change the visible map limits
-
- var westedge  = -11.0;          // Minimum longitude (degrees)
- var eastedge  =   2.0;          // Maximum longitude (degrees)
- var southedge =  49.5;          // Minimum latitude (degrees)
- var northedge =  61.0;          // Maximum latitude (degrees)
- var zoomout   =     4;          // Minimum zoom
- var zoomin    =    15;          // Maximum zoom
-
- // EDIT THIS above to change the visible map limits
+ // Map properties (North/South and East/West limits and zoom in/out limits) are now in mapprops.js
+ // Map URLs are now in mapprops.js
 
  //
  // Create the map
@@ -561,30 +473,33 @@ function map_init(lat,lon,zoom)
                             projection: epsg900913,
                             displayProjection: epsg4326,
 
-                            minZoomLevel: zoomout,
-                            numZoomLevels: zoomin-zoomout+1,
-                            maxResolution: 156543.0339 / Math.pow(2,zoomout),
+                            minZoomLevel: mapprops.zoomout,
+                            numZoomLevels: mapprops.zoomin-mapprops.zoomout+1,
+                            maxResolution: 156543.0339 / Math.pow(2,mapprops.zoomout),
 
                             maxExtent:        new OpenLayers.Bounds(-20037508.34, -20037508.34, 20037508.34, 20037508.34),
-                            restrictedExtent: new OpenLayers.Bounds(westedge,southedge,eastedge,northedge).transform(epsg4326,epsg900913),
+                            restrictedExtent: new OpenLayers.Bounds(mapprops.westedge,mapprops.southedge,mapprops.eastedge,mapprops.northedge).transform(epsg4326,epsg900913),
 
                             units: "m"
                            });
 
  map.events.register("moveend", map, mapMoved);
 
- // Add a map tile layer (OpenStreetMap tiles, direct access)
+ // Add map tile layers
 
- layerMapOSM = new OpenLayers.Layer.TMS("Original OSM map",
-                                        "http://tile.openstreetmap.org/",
-                                        {
-                                         emptyUrl: "http://openstreetmap.org/openlayers/img/404.png",
-                                         type: 'png',
-                                         getURL: limitedUrl,
-                                         displayOutsideMaxExtent: true,
-                                         buffer: 1
-                                        });
- map.addLayer(layerMapOSM);
+ for(var l=0;l < mapprops.mapdata.length;l++)
+   {
+    layerMap[l] = new OpenLayers.Layer.TMS(mapprops.mapdata[l].label,
+                                           mapprops.mapdata[l].baseurl,
+                                           {
+                                            emptyUrl: mapprops.mapdata[l].errorurl,
+                                            type: 'png',
+                                            getURL: limitedUrl,
+                                            displayOutsideMaxExtent: true,
+                                            buffer: 1
+                                           });
+    map.addLayer(layerMap[l]);
+   }
 
  // Get a URL for the tile; limited to map restricted extent.
 
@@ -690,6 +605,15 @@ function map_init(lat,lon,zoom)
 
  if(lon != 'lon' && lat != 'lat' && zoom != 'zoom')
    {
+    if(lon<mapprops.westedge) lon=mapprops.westedge;
+    if(lon>mapprops.eastedge) lon=mapprops.eastedge;
+
+    if(lat<mapprops.southedge) lat=mapprops.southedge;
+    if(lat>mapprops.northedge) lat=mapprops.northedge;
+
+    if(zoom<mapprops.zoomout) zoom=mapprops.zoomout;
+    if(zoom>mapprops.zoomin)  zoom=mapprops.zoomin;
+
     var lonlat = new OpenLayers.LonLat(lon,lat).transform(epsg4326,map.getProjectionObject());
 
     map.moveTo(lonlat,zoom-map.minZoomLevel);
@@ -1283,9 +1207,11 @@ function runRouterSuccess(response)
  var lines=response.responseText.split('\n');
 
  var uuid=lines[0];
- var cpuinfo=lines[1];
- var distinfo=lines[2];
- var message=lines[3];
+ var cpuinfo=lines[1];  // not used
+ var distinfo=lines[2]; // not used
+ var message=lines[3];  // content not used
+
+ var link;
 
  // Update the status message
 
@@ -1293,20 +1219,23 @@ function runRouterSuccess(response)
    {
     displayStatus("result","error");
     hideshow_show('help_route');
+
+    link=document.getElementById("router_log_error");
+    link.href="results.cgi?uuid=" + uuid + ";type=router;format=log";
+
     return;
    }
  else
    {
     displayStatus("result","complete");
     hideshow_hide('help_route');
+
+    link=document.getElementById("router_log_complete");
+    link.href="results.cgi?uuid=" + uuid + ";type=router;format=log";
    }
 
  // Update the routing result message
 
- displayStatus(routing_type,"info",distinfo.bold());
-
- var link;
-
  link=document.getElementById(routing_type + "_html");
  link.href="results.cgi?uuid=" + uuid + ";type=" + routing_type + ";format=html";
  link=document.getElementById(routing_type + "_gpx_track");
@@ -1369,12 +1298,12 @@ function displayStatus(type,subtype,content)
    }
  while(child != undefined);
 
- var span_status=document.getElementById(type + "_status_" + subtype);
+ var chosen_status=document.getElementById(type + "_status_" + subtype);
 
- span_status.style.display="";
+ chosen_status.style.display="";
 
  if(content != null)
-    span_status.innerHTML=content;
+    chosen_status.innerHTML=content;
 }
 
 
@@ -1474,6 +1403,8 @@ function getRouteSuccess(response)
       }
    }
 
+ displayStatus(routing_type,"info",points[point-1].total.bold());
+
  var result="<table onmouseout='highlight(\"" + routing_type + "\",-1)'>";
 
  for(var p=0;p<point-1;p++)
