@@ -79,7 +79,7 @@ var turn_restriction_style;
 ////////////////////////////////////////////////////////////////////////////////
 
 var map;
-var layerMapOSM, layerVectors, layerBoxes;
+var layerMap=[], layerVectors, layerBoxes;
 var epsg4326, epsg900913;
 var map_args;
 
@@ -91,21 +91,8 @@ var box;
 
 function map_init(lat,lon,zoom)
 {
- // Default configuration:
- // UK coordinate range
- // West -11.0, South 49.5, East 2.0, North 61.0
- // Zoom level 4 to 15
-
- // EDIT THIS below to change the visible map limits
-
- var westedge  = -11.0;          // Minimum longitude (degrees)
- var eastedge  =   2.0;          // Maximum longitude (degrees)
- var southedge =  49.5;          // Minimum latitude (degrees)
- var northedge =  61.0;          // Maximum latitude (degrees)
- var zoomout   =     4;          // Minimum zoom
- var zoomin    =    15;          // Maximum zoom
-
- // EDIT THIS above to change the visible map limits
+ // Map properties (North/South and East/West limits and zoom in/out limits) are now in mapprops.js
+ // Map URLs are now in mapprops.js
 
  //
  // Create the map
@@ -126,30 +113,33 @@ function map_init(lat,lon,zoom)
                             projection: epsg900913,
                             displayProjection: epsg4326,
 
-                            minZoomLevel: zoomout,
-                            numZoomLevels: zoomin-zoomout+1,
-                            maxResolution: 156543.0339 / Math.pow(2,zoomout),
+                            minZoomLevel: mapprops.zoomout,
+                            numZoomLevels: mapprops.zoomin-mapprops.zoomout+1,
+                            maxResolution: 156543.0339 / Math.pow(2,mapprops.zoomout),
 
                             maxExtent:        new OpenLayers.Bounds(-20037508.34, -20037508.34, 20037508.34, 20037508.34),
-                            restrictedExtent: new OpenLayers.Bounds(westedge,southedge,eastedge,northedge).transform(epsg4326,epsg900913),
+                            restrictedExtent: new OpenLayers.Bounds(mapprops.westedge,mapprops.southedge,mapprops.eastedge,mapprops.northedge).transform(epsg4326,epsg900913),
 
                             units: "m"
                            });
 
  map.events.register("moveend", map, mapMoved);
 
- // Add a map tile layer (OpenStreetMap tiles, direct access)
+ // Add map tile layers
 
- layerMapOSM = new OpenLayers.Layer.TMS("Original OSM map",
-                                        "http://tile.openstreetmap.org/",
-                                        {
-                                         emptyUrl: "http://openstreetmap.org/openlayers/img/404.png",
-                                         type: 'png',
-                                         getURL: limitedUrl,
-                                         displayOutsideMaxExtent: true,
-                                         buffer: 1
-                                        });
- map.addLayer(layerMapOSM);
+ for(var l=0;l < mapprops.mapdata.length;l++)
+   {
+    layerMap[l] = new OpenLayers.Layer.TMS(mapprops.mapdata[l].label,
+                                           mapprops.mapdata[l].baseurl,
+                                           {
+                                            emptyUrl: mapprops.mapdata[l].errorurl,
+                                            type: 'png',
+                                            getURL: limitedUrl,
+                                            displayOutsideMaxExtent: true,
+                                            buffer: 1
+                                           });
+    map.addLayer(layerMap[l]);
+   }
 
  // Get a URL for the tile; limited to map restricted extent.
 
@@ -205,6 +195,15 @@ function map_init(lat,lon,zoom)
 
  if(lon != 'lon' && lat != 'lat' && zoom != 'zoom')
    {
+    if(lon<mapprops.westedge) lon=mapprops.westedge;
+    if(lon>mapprops.eastedge) lon=mapprops.eastedge;
+
+    if(lat<mapprops.southedge) lat=mapprops.southedge;
+    if(lat>mapprops.northedge) lat=mapprops.northedge;
+
+    if(zoom<mapprops.zoomout) zoom=mapprops.zoomout;
+    if(zoom>mapprops.zoomin)  zoom=mapprops.zoomin;
+
     var lonlat = new OpenLayers.LonLat(lon,lat).transform(epsg4326,map.getProjectionObject());
 
     map.moveTo(lonlat,zoom-map.minZoomLevel);
@@ -250,6 +249,38 @@ function updateCustomURL()
 /////////////////////////////// Server handling ////////////////////////////////
 ////////////////////////////////////////////////////////////////////////////////
 
+//
+// Display the status
+//
+
+function displayStatus(type,subtype,content)
+{
+ var div_status=document.getElementById("result_status");
+
+ var child=div_status.firstChild;
+
+ do
+   {
+    if(child.id != undefined)
+       child.style.display="none";
+
+    child=child.nextSibling;
+   }
+ while(child != undefined);
+
+ var chosen_status=document.getElementById("result_status_" + type);
+
+ chosen_status.style.display="";
+
+ if(subtype != null)
+   {
+    var format_status=document.getElementById("result_status_" + subtype).innerHTML;
+
+    chosen_status.innerHTML=format_status.replace('#',String(content));
+   }
+}
+
+
 //
 // Display data statistics
 //
@@ -299,8 +330,7 @@ function displayData(datatype)
 
  // Print the status
 
- var div_status=document.getElementById("result_status");
- div_status.innerHTML = "No data displayed";
+ displayStatus("no_data");
 
  // Return if just here to clear the data
 
@@ -320,10 +350,6 @@ function displayData(datatype)
  url=url + ";latmax=" + mapbounds.top;
  url=url + ";data=" + datatype;
 
- // Print the status
-
- div_status.innerHTML = "Fetching " + datatype + " data ...";
-
  // Use AJAX to get the data
 
  switch(datatype)
@@ -359,10 +385,6 @@ function runJunctionsSuccess(response)
 {
  var lines=response.responseText.split('\n');
 
-// This won't update the browser window
-// var div_status=document.getElementById("result_status");
-// div_status.innerHTML = "Processing " + (lines.length-2) + " junctions ...";
-
  var features=[];
 
  for(var line=0;line<lines.length;line++)
@@ -398,8 +420,7 @@ function runJunctionsSuccess(response)
 
  layerVectors.addFeatures(features);
 
- var div_status=document.getElementById("result_status");
- div_status.innerHTML = "Processed " + (lines.length-2) + " junctions";
+ displayStatus("data","junctions",lines.length-2);
 }
 
 
@@ -411,10 +432,6 @@ function runSuperSuccess(response)
 {
  var lines=response.responseText.split('\n');
 
-// This won't update the browser window
-// var div_status=document.getElementById("result_status");
-// div_status.innerHTML = "Processing " + (lines.length-2) + " super-nodes/segments ...";
-
  var features=[];
 
  var nodepoint;
@@ -463,8 +480,7 @@ function runSuperSuccess(response)
 
  layerVectors.addFeatures(features);
 
- var div_status=document.getElementById("result_status");
- div_status.innerHTML = "Processed " + (lines.length-2) + " super-nodes/segments";
+ displayStatus("data","super",lines.length-2);
 }
 
 
@@ -476,10 +492,6 @@ function runOnewaySuccess(response)
 {
  var lines=response.responseText.split('\n');
 
-// This won't update the browser window
-// var div_status=document.getElementById("result_status");
-// div_status.innerHTML = "Processing " + (lines.length-2) + " oneway segments ...";
-
  var features=[];
 
  for(var line=0;line<lines.length;line++)
@@ -535,8 +547,7 @@ function runOnewaySuccess(response)
 
  layerVectors.addFeatures(features);
 
- var div_status=document.getElementById("result_status");
- div_status.innerHTML = "Processed " + (lines.length-2) + " oneway segments";
+ displayStatus("data","oneway",lines.length-2);
 }
 
 
@@ -548,10 +559,6 @@ function runTurnsSuccess(response)
 {
  var lines=response.responseText.split('\n');
 
-// This won't update the browser window
-// var div_status=document.getElementById("result_status");
-// div_status.innerHTML = "Processing " + (lines.length-2) + " turn restrictions ...";
-
  var features=[];
 
  for(var line=0;line<lines.length;line++)
@@ -596,8 +603,7 @@ function runTurnsSuccess(response)
 
  layerVectors.addFeatures(features);
 
- var div_status=document.getElementById("result_status");
- div_status.innerHTML = "Processed " + (lines.length-2) + " turn restrictions";
+ displayStatus("data","turns",lines.length-2);
 }
 
 
@@ -609,10 +615,6 @@ function runLimitSuccess(response)
 {
  var lines=response.responseText.split('\n');
 
-// This won't update the browser window
-// var div_status=document.getElementById("result_status");
-// div_status.innerHTML = "Processing " + (lines.length-2) + " limits ...";
-
  var features=[];
 
  var nodelonlat;
@@ -669,8 +671,7 @@ function runLimitSuccess(response)
 
  layerVectors.addFeatures(features);
 
- var div_status=document.getElementById("result_status");
- div_status.innerHTML = "Processed " + (lines.length-2) + " limits";
+ displayStatus("data","limit",lines.length-2);
 }
 
 
@@ -680,8 +681,5 @@ function runLimitSuccess(response)
 
 function runFailure(response)
 {
- var div_status=document.getElementById("result_status");
- div_status.innerHTML = "Failed to get visualiser data!";
-
- window.alert("Failed to get visualiser data!\n" + response.statusText);
+ displayStatus("error");
 }
